<?php

namespace Phppot\SecurityService;

class AntiCSRFService
{

    private $formTokenKey = 'phppot-csrf-token-label';
    private $sessionTokenKey = 'PHPPOT_CSRF_TOKEN_SESS_IDX';


    public function __construct()
    {
        if (empty($_SESSION)) {
            throw new \Error('No session available for persistence');
        }
    }

    /**
     * Insert a CSRF token to a form
     */
    public function insertHiddenToken()
    {
        $csrfToken = $this->getCSRFToken();

        echo "<!--\n--><input type=\"hidden\"" .
            " name=\"" . $this->formTokenKey . "\"" .
            " value=\"" . $csrfToken . "\"" . " />";
    }

    /**
     * Generate, store, and return the CSRF token
     *
     * @return string[]
     */
    public function getCSRFToken()
    {
        if (empty($_SESSION[$this->sessionTokenKey])) {
            $newToken = bin2hex(openssl_random_pseudo_bytes(32));
            $_SESSION[$this->sessionTokenKey] = $newToken;
        }

        $token = $_SESSION[$this->sessionTokenKey];

        return $token;
    }

    /**
     * the actual validation of CSRF happens here and returns boolean
     *
     * @return boolean
     */
    public function isValidRequest()
    {
        $isValid = false;
        if (!empty($_POST)) {
            $isValid = $this->validateRequest();
        }
        return $isValid;
    }

    /**
     * Validate a request based on session
     *
     * @return bool
     */
    public function validateRequest()
    {
        if (!isset($_SESSION[$this->sessionTokenKey])) {
            // CSRF Token not found in the session
            return false;
        }

        if (!empty($_POST[$this->formTokenKey])) {
            // Let's pull the POST data
            $token = $_POST[$this->formTokenKey];
        } else {
            return false;
        }

        if (!\is_string($token)) {
            return false;
        }

        $expected = $_SESSION[$this->sessionTokenKey];

        return \hash_equals($expected, $token);
    }
}
